#include "cfglr_log.h"
#include "cfglr_signaler_lvgl.h"
#include "lvgl.h"

#define TAG "CFGLR_SIG_LVGL"

void cfglr_signaler_lvgl_change_cb(lv_event_t *e) {
  lv_obj_t *target = lv_event_get_current_target(e);
  cfglr_element_t *element = lv_event_get_user_data(e);

  if(lv_obj_check_type(target, &lv_checkbox_class)) {
    lv_state_t state = lv_obj_get_state(target);

    uint8_t mon = *(int*)element->data;
    // A new event is pushed when dispatching so the loop needs to be broken if the value
    // didn't actually change.
    if(mon != (state & LV_STATE_CHECKED)) {
      CFGLR_LOGD(TAG, "element->data: %p %d", element->data, *(uint8_t*)element->data);

      *(int*)(element->data) = (state & LV_STATE_CHECKED);
      cfglr_commit(element);
    }
  }
}

uint8_t cfglr_signaler_lvgl_init(cfglr_signaler_t *signaler, cfglr_element_t *element, cfglr_handle_t *handle) {
  CFGLR_LOGI(TAG, "init");

  CFGLR_LOGI(TAG, "ui handle: %p", signaler->handle);
  lv_obj_add_event_cb((lv_obj_t*)signaler->handle, cfglr_signaler_lvgl_change_cb, LV_EVENT_VALUE_CHANGED, element);

  return 0;
}

uint8_t cfglr_signaler_lvgl_dispatch(cfglr_signaler_t *signaler, cfglr_element_t *element, cfglr_signal_e sig) {
  CFGLR_LOGI(TAG, "dispatch: %d", *(uint8_t*)element->data);

  if(strstr(element->key, "monitor") != NULL) {
    if(*(uint8_t*)element->data == 1) {
      lv_obj_add_state((lv_obj_t*)signaler->handle, LV_STATE_CHECKED);
    } else {
      lv_obj_clear_state((lv_obj_t*)signaler->handle, LV_STATE_CHECKED);
    }

    lv_event_send((lv_obj_t*)signaler->handle, LV_EVENT_VALUE_CHANGED, NULL);
  }

  return 0;
}
